@@ -1,11 +1,11 @@
-# coding: utf8
-from __future__ import unicode_literals
-
 import re
 
+from ..tokenizer_exceptions import BASE_EXCEPTIONS
 from .punctuation import ELISION, HYPHENS
 from ..char_classes import ALPHA_LOWER, ALPHA
-from ...symbols import ORTH, LEMMA
+from ...symbols import ORTH
+from ...util import update_exc
+
 
 # not using the large _tokenizer_exceptions_list by default as it slows down the tokenizer
 # from ._tokenizer_exceptions_list import FR_BASE_EXCEPTIONS
@@ -28,29 +28,29 @@ def lower_first_letter(text):
     return text[0].lower() + text[1:]
 
 
-_exc = {"J.-C.": [{LEMMA: "Jésus", ORTH: "J."}, {LEMMA: "Christ", ORTH: "-C."}]}
+_exc = {"J.-C.": [{ORTH: "J."}, {ORTH: "-C."}]}
 
 
 for exc_data in [
-    {LEMMA: "avant", ORTH: "av."},
-    {LEMMA: "janvier", ORTH: "janv."},
-    {LEMMA: "février", ORTH: "févr."},
-    {LEMMA: "avril", ORTH: "avr."},
-    {LEMMA: "juillet", ORTH: "juill."},
-    {LEMMA: "septembre", ORTH: "sept."},
-    {LEMMA: "octobre", ORTH: "oct."},
-    {LEMMA: "novembre", ORTH: "nov."},
-    {LEMMA: "décembre", ORTH: "déc."},
-    {LEMMA: "après", ORTH: "apr."},
-    {LEMMA: "docteur", ORTH: "Dr."},
-    {LEMMA: "monsieur", ORTH: "M."},
-    {LEMMA: "monsieur", ORTH: "Mr."},
-    {LEMMA: "madame", ORTH: "Mme."},
-    {LEMMA: "mademoiselle", ORTH: "Mlle."},
-    {LEMMA: "numéro", ORTH: "n°"},
-    {LEMMA: "degrés", ORTH: "d°"},
-    {LEMMA: "saint", ORTH: "St."},
-    {LEMMA: "sainte", ORTH: "Ste."},
+    {ORTH: "av."},
+    {ORTH: "janv."},
+    {ORTH: "févr."},
+    {ORTH: "avr."},
+    {ORTH: "juill."},
+    {ORTH: "sept."},
+    {ORTH: "oct."},
+    {ORTH: "nov."},
+    {ORTH: "déc."},
+    {ORTH: "apr."},
+    {ORTH: "Dr."},
+    {ORTH: "M."},
+    {ORTH: "Mr."},
+    {ORTH: "Mme."},
+    {ORTH: "Mlle."},
+    {ORTH: "n°"},
+    {ORTH: "d°"},
+    {ORTH: "St."},
+    {ORTH: "Ste."},
 ]:
     _exc[exc_data[ORTH]] = [exc_data]
 
@@ -80,55 +80,37 @@ for orth in [
     _exc[orth] = [{ORTH: orth}]
 
 
-for verb, verb_lemma in [
-    ("a", "avoir"),
-    ("est", "être"),
-    ("semble", "sembler"),
-    ("indique", "indiquer"),
-    ("moque", "moquer"),
-    ("passe", "passer"),
+for verb in [
+    "a",
+    "est" "semble",
+    "indique",
+    "moque",
+    "passe",
 ]:
     for orth in [verb, verb.title()]:
         for pronoun in ["elle", "il", "on"]:
-            token = "{}-t-{}".format(orth, pronoun)
-            _exc[token] = [
-                {LEMMA: verb_lemma, ORTH: orth},  # , TAG: "VERB"},
-                {LEMMA: "t", ORTH: "-t"},
-                {LEMMA: pronoun, ORTH: "-" + pronoun},
-            ]
+            token = f"{orth}-t-{pronoun}"
+            _exc[token] = [{ORTH: orth}, {ORTH: "-t"}, {ORTH: "-" + pronoun}]
 
-for verb, verb_lemma in [("est", "être")]:
+for verb in ["est"]:
     for orth in [verb, verb.title()]:
-        token = "{}-ce".format(orth)
-        _exc[token] = [
-            {LEMMA: verb_lemma, ORTH: orth},  # , TAG: "VERB"},
-            {LEMMA: "ce", ORTH: "-ce"},
-        ]
+        _exc[f"{orth}-ce"] = [{ORTH: orth}, {ORTH: "-ce"}]
 
 
-for pre, pre_lemma in [("qu'", "que"), ("n'", "ne")]:
+for pre in ["qu'", "n'"]:
     for orth in [pre, pre.title()]:
-        _exc["%sest-ce" % orth] = [
-            {LEMMA: pre_lemma, ORTH: orth},
-            {LEMMA: "être", ORTH: "est"},
-            {LEMMA: "ce", ORTH: "-ce"},
-        ]
+        _exc[f"{orth}est-ce"] = [{ORTH: orth}, {ORTH: "est"}, {ORTH: "-ce"}]
 
 
 for verb, pronoun in [("est", "il"), ("EST", "IL")]:
-    token = "{}-{}".format(verb, pronoun)
-    _exc[token] = [
-        {LEMMA: "être", ORTH: verb},
-        {LEMMA: pronoun, ORTH: "-" + pronoun},
-    ]
+    _exc[f"{verb}-{pronoun}"] = [{ORTH: verb}, {ORTH: "-" + pronoun}]
 
 
 for s, verb, pronoun in [("s", "est", "il"), ("S", "EST", "IL")]:
-    token = "{}'{}-{}".format(s, verb, pronoun)
-    _exc[token] = [
-        {LEMMA: "se", ORTH: s + "'"},
-        {LEMMA: "être", ORTH: verb},
-        {LEMMA: pronoun, ORTH: "-" + pronoun},
+    _exc[f"{s}'{verb}-{pronoun}"] = [
+        {ORTH: s + "'"},
+        {ORTH: verb},
+        {ORTH: "-" + pronoun},
     ]
 
 
@@ -455,7 +437,7 @@ _regular_exp += [
 ]
 
 
-TOKENIZER_EXCEPTIONS = _exc
+TOKENIZER_EXCEPTIONS = update_exc(BASE_EXCEPTIONS, _exc)
 TOKEN_MATCH = re.compile(
     "(?iu)" + "|".join("(?:{})".format(m) for m in _regular_exp)
 ).match
