@@ -1,11 +1,7 @@
-# coding: utf-8
-from __future__ import unicode_literals
-
 import pytest
 from spacy.matcher import Matcher
-from spacy.matcher._schemas import TOKEN_PATTERN_SCHEMA
 from spacy.errors import MatchPatternError
-from spacy.util import get_json_validator, validate_json
+from spacy.schemas import validate_token_pattern
 
 # (pattern, num errors with validation, num errors identified with minimal
 #  checks)
@@ -18,12 +14,12 @@ TEST_PATTERNS = [
     ('[{"TEXT": "foo"}, {"LOWER": "bar"}]', 1, 1),
     ([1, 2, 3], 3, 1),
     # Bad patterns flagged outside of Matcher
-    ([{"_": {"foo": "bar", "baz": {"IN": "foo"}}}], 1, 0),
+    ([{"_": {"foo": "bar", "baz": {"IN": "foo"}}}], 2, 0),  # prev: (1, 0)
     # Bad patterns not flagged with minimal checks
     ([{"LENGTH": "2", "TEXT": 2}, {"LOWER": "test"}], 2, 0),
-    ([{"LENGTH": {"IN": [1, 2, "3"]}}, {"POS": {"IN": "VERB"}}], 2, 0),
-    ([{"LENGTH": {"VALUE": 5}}], 1, 0),
-    ([{"TEXT": {"VALUE": "foo"}}], 1, 0),
+    ([{"LENGTH": {"IN": [1, 2, "3"]}}, {"POS": {"IN": "VERB"}}], 4, 0),  # prev: (2, 0)
+    ([{"LENGTH": {"VALUE": 5}}], 2, 0),  # prev: (1, 0)
+    ([{"TEXT": {"VALUE": "foo"}}], 2, 0),  # prev: (1, 0)
     ([{"IS_DIGIT": -1}], 1, 0),
     ([{"ORTH": -1}], 1, 0),
     # Good patterns
@@ -34,17 +30,11 @@ TEST_PATTERNS = [
     ([{"LOWER": {"REGEX": "^X", "NOT_IN": ["XXX", "XY"]}}], 0, 0),
     ([{"NORM": "a"}, {"POS": {"IN": ["NOUN"]}}], 0, 0),
     ([{"_": {"foo": {"NOT_IN": ["bar", "baz"]}, "a": 5, "b": {">": 10}}}], 0, 0),
+    ([{"orth": "foo"}], 0, 0),  # prev: xfail
     ([{"IS_SENT_START": True}], 0, 0),
     ([{"SENT_START": True}], 0, 0),
 ]
 
-XFAIL_TEST_PATTERNS = [([{"orth": "foo"}], 0, 0)]
-
-
-@pytest.fixture
-def validator():
-    return get_json_validator(TOKEN_PATTERN_SCHEMA)
-
 
 @pytest.mark.parametrize(
     "pattern", [[{"XX": "y"}, {"LENGTH": "2"}, {"TEXT": {"IN": 5}}]]
@@ -56,15 +46,8 @@ def test_matcher_pattern_validation(en_vocab, pattern):
 
 
 @pytest.mark.parametrize("pattern,n_errors,_", TEST_PATTERNS)
-def test_pattern_validation(validator, pattern, n_errors, _):
-    errors = validate_json(pattern, validator)
-    assert len(errors) == n_errors
-
-
-@pytest.mark.xfail
-@pytest.mark.parametrize("pattern,n_errors,_", XFAIL_TEST_PATTERNS)
-def test_xfail_pattern_validation(validator, pattern, n_errors, _):
-    errors = validate_json(pattern, validator)
+def test_pattern_validation(pattern, n_errors, _):
+    errors = validate_token_pattern(pattern)
     assert len(errors) == n_errors
 
 
@@ -78,10 +61,10 @@ def test_minimal_pattern_validation(en_vocab, pattern, n_errors, n_min_errors):
         matcher.add("TEST", [pattern])
 
 
-def test_pattern_warnings(en_vocab):
+def test_pattern_errors(en_vocab):
     matcher = Matcher(en_vocab)
     # normalize "regex" to upper like "text"
     matcher.add("TEST1", [[{"text": {"regex": "regex"}}]])
-    # warn if subpattern attribute isn't recognized and processed
-    with pytest.warns(UserWarning):
+    # error if subpattern attribute isn't recognized and processed
+    with pytest.raises(MatchPatternError):
         matcher.add("TEST2", [[{"TEXT": {"XX": "xx"}}]])
